package com.loafle.overflow.module.generator.service;

import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.crawler.config.Connection;
import com.loafle.overflow.crawler.config.Crawler;
import com.loafle.overflow.crawler.config.Schedule;
import com.loafle.overflow.module.infra.model.Infra;
import com.loafle.overflow.module.meta.model.MetaSensorItemKey;
import com.loafle.overflow.module.meta.type.MetaCrawlerEnum;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("InfraServiceGenerator")
public class InfraServiceGenerator {

    @Autowired
    private GenerateUtil generateUtil;

    @Autowired
    private InfraServiceMysqlGenerator infraServiceMysqlGenerator;

    public String process(Sensor dbSensor, List<SensorItem> sensorItems, Infra infra) throws IOException {
        com.loafle.overflow.module.infra.model.InfraService infraService = (com.loafle.overflow.module.infra.model.InfraService)infra;

        Config config = new Config();
        config.setId(String.valueOf(dbSensor.getId()));

        com.loafle.overflow.crawler.config.Target target = new com.loafle.overflow.crawler.config.Target();
        Connection connection = new Connection();
        connection.setIp(StringConvertor.intToIp(infraService.getHost().getIp()));
        connection.setPort(String.valueOf(infraService.getPort()));
        connection.setPortType(infraService.getPortType());
        connection.setSsl(infraService.isTlsType());

        target.setConnection(connection);

        Map<String, Object> auth = this.generateUtil.getAuth(dbSensor.getCrawler(), infraService);

        target.setAuth(auth);

        config.setTarget(target);

        // FIXME: Interval
        Schedule schedule = new Schedule();
        schedule.setInterval("5");
        config.setSchedule(schedule);

        Crawler crawler = this.generateUtil.getCrawler(dbSensor.getCrawler());
        config.setCrawler(crawler);

        Map<Integer, MetaSensorItemKey> keyMap = this.generateUtil.initMappingMap(dbSensor.getCrawler());

        if(dbSensor.getCrawler().getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
            this.infraServiceMysqlGenerator.process(sensorItems, keyMap, dbSensor, config);
        }

        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.writeValueAsString(config);
    }
}
